"""
Base interface for all API interfaces.
"""

from logging import Logger

from pyswx.logger import log


class BaseInterface:
    """
    Base interface for all api interfaces.
    This class provides a logger property that can be used by derived classes.
    Logs to 'pyswx'.
    """

    def __init__(self) -> None:
        pass

    @property
    def logger(self) -> Logger:
        return log.logger



"""
swOpenDocOptions_e Enumeration

Reference:
https://help.solidworks.com/2024/english/api/swconst/SOLIDWORKS.Interop.swconst~SOLIDWORKS.Interop.swconst.swOpenDocOptions_e.html?id=9bba448a555b46fc8ee9e9abae302e42#Pg0

Status: 🟢
"""

from enum import IntEnum


class SWOpenDocOptionsE(IntEnum):
    SW_OPEN_DOC_SILENT = 1  #: Open document silently
    SW_OPEN_DOC_READ_ONLY = 2  #: Open document read-only
    SW_OPEN_DOC_VIEW_ONLY = 4  #: Open document in Large Design Review mode (assemblies only)
    SW_OPEN_DOC_LOAD_MODEL = 16  #: Load detached model upon opening document (drawings only)
    SW_OPEN_DOC_OPEN_DETAILING_MODE = 1024  #: Open document in detailing mode
    SW_OPEN_DOC_LOAD_LIGHTWEIGHT = 128  #: Open assembly document as lightweight
    SW_OPEN_DOC_LOAD_EXTERNAL_REFERENCES_IN_MEMORY = 512  #: Open external references in memory only
    SW_OPEN_DOC_OVERRIDE_DEFAULT_LOAD_LIGHTWEIGHT = (
        64  #: Override default setting whether to open an assembly document as lightweight
    )
    SW_OPEN_DOC_LDR_EDIT_ASSEMBLY = 2048  #: Open in Large Design Review (resolved) mode with edit assembly enabled; use in combination with SW_OPEN_DOC_VIEW_ONLY
    SW_OPEN_DOC_SPEEDPAK = 4096  #: Open document using the SpeedPak option
    SW_OPEN_DOC_RAPID_DRAFT = 8  #: Convert document to Detached format (drawings only)
    SW_OPEN_DOC_ADVANCED_CONFIG = 8192  #: Open assembly using an advanced configuration
    SW_OPEN_DOC_AUTO_MISSING_CONFIG = 32  #: Obsolete; do not use



# Filename: sw_body_type_e.py

"""
swBodyType_e Enumeration

Reference:
https://help.solidworks.com/2024/english/api/swconst/SOLIDWORKS.Interop.swconst~SOLIDWORKS.Interop.swconst.swBodyType_e.html

Status: 🟢
"""

from enum import IntEnum


class SWBodyTypeE(IntEnum):
    """Defines valid types of bodies in SolidWorks models."""

    SW_SOLID_BODY = 0  # Solid body
    SW_SHEET_BODY = 1  # Sheet body
    SW_WIRE_BODY = 2  # Wire body
    SW_MINIMUM_BODY = 3  # Point body
    SW_GENERAL_BODY = 4  # General, nonmanifold body
    SW_EMPTY_BODY = 5  # NULL body
    SW_MESH_BODY = 6  # Mesh body
    SW_GRAPHICS_BODY = 7  # Graphics body
    SW_ALL_BODIES = -1  # All solid and sheet bodies



"""
swSaveItemsPathError_e Enumeration

Reference:
https://help.solidworks.com/2024/english/api/swconst/SOLIDWORKS.Interop.swconst~SOLIDWORKS.Interop.swconst.swSaveItemsPathError_e.html

Status: 🟢
"""

from enum import IntEnum


class SWSaveItemsPathErrorE(IntEnum):
    """Error return codes for ModifyItemsNameAndPath."""

    SW_SAVE_ITEMS_PATH_ERROR_SUCCEEDED = 0  # Operation succeeded
    SW_SAVE_ITEMS_PATH_ERROR_ARRAY_SIZE_NOT_MATCHING = 1  # Input arrays must be the same size
    SW_SAVE_ITEMS_PATH_ERROR_INVALID_PATH = 2  # Invalid path or no write access
    SW_SAVE_ITEMS_PATH_ERROR_WRONG_COMPONENT_NAME = 3  # Unsupported component name



"""
swFileLoadError_e Enumeration

Reference:
https://help.solidworks.com/2024/english/api/swconst/SOLIDWORKS.Interop.swconst~SOLIDWORKS.Interop.swconst.swFileLoadError_e.html

Status: 🟢
"""

from enum import IntEnum


class SWFileLoadErrorE(IntEnum):
    """File load errors. Bitmask."""

    SW_GENERIC_ERROR = 1  # Another error was encountered
    SW_FILE_NOT_FOUND_ERROR = (
        2  # Unable to locate the file; the file is not loaded or the referenced file is suppressed
    )
    SW_INVALID_FILE_TYPE_ERROR = 1024  # File type argument is not valid
    SW_FUTURE_VERSION = 8192  # The document was saved in a future version of SOLIDWORKS
    SW_FILE_WITH_SAME_TITLE_ALREADY_OPEN = 65536  # A document with the same name is already open
    SW_LIQUID_MACHINE_DOC = 131072  # File encrypted by Liquid Machines
    SW_LOW_RESOURCES_ERROR = 262144  # File is open and blocked because the system memory is low, or the number of GDI handles has exceeded the allowed maximum
    SW_NO_DISPLAY_DATA = 524288  # File contains no display data
    SW_FILE_REQUIRES_REPAIR_ERROR = 2097152  # A document has non-critical custom property data corruption
    SW_FILE_CRITICAL_DATA_REPAIR_ERROR = 4194304  # A document has critical data corruption
    SW_ADDIN_INTERRUPT_ERROR = 1048576  # The user attempted to open a file, and then interrupted the open-file routine to open a different file
    SW_APPLICATION_BUSY = 8388608  # File open is blocked when application is busy
    SW_CONNECTED_IS_OFFLINE = 16777216  # Connected is offline



"""
IComponent2 Interface Members

Reference:
https://help.solidworks.com/2024/english/api/sldworksapi/SolidWorks.Interop.sldworks~SolidWorks.Interop.sldworks.IComponent2_members.html

Status: 🔴
"""

from pathlib import Path
from typing import TYPE_CHECKING
from typing import List

from pythoncom import VT_ARRAY
from pythoncom import VT_BOOL
from pythoncom import VT_BSTR
from pythoncom import VT_DISPATCH
from pythoncom import VT_I4
from pythoncom import VT_NULL
from pythoncom import VT_R8
from win32com.client import VARIANT

from pyswx.api.base_interface import BaseInterface
from pyswx.api.sldworks.interfaces.i_custom_property_manager import ICustomPropertyManager
from pyswx.api.sldworks.interfaces.i_enum_bodies_2 import IEnumBodies2
from pyswx.api.sldworks.interfaces.i_math_transform import IMathTransform
from pyswx.api.swconst.enumerations import SWBodyTypeE
from pyswx.api.swconst.enumerations import SWComponentSolvingOptionE
from pyswx.api.swconst.enumerations import SWComponentSuppressionStateE
from pyswx.api.swconst.enumerations import SWDocumentTypesE
from pyswx.api.swconst.enumerations import SWInConfigurationOptsE

if TYPE_CHECKING:
    from pyswx.api.sldworks.interfaces.i_model_doc_2 import IModelDoc2


class IComponent2(BaseInterface):
    def __init__(self, com_object):
        super().__init__()
        self.com_object = com_object

    def __repr__(self) -> str:
        return f"IComponent2({self.com_object})"

    @property
    def component_reference(self) -> str:
        """
        Gets the component reference for this component.

        Reference:
        https://help.solidworks.com/2024/english/api/sldworksapi/SolidWorks.Interop.sldworks~SolidWorks.Interop.sldworks.IComponent2~ComponentReference.html
        """
        return self.com_object.ComponentReference

    @component_reference.setter
    def component_reference(self, value: str):
        self.com_object.ComponentReference = value

    def custom_property_manager(self, config_name: str) -> ICustomPropertyManager:
        """
        Gets the custom property information for this configuration.

        Reference:
        https://help.solidworks.com/2024/english/api/sldworksapi/SolidWorks.Interop.sldworks~SolidWorks.Interop.sldworks.IComponent2~CustomPropertyManager.html

        Note:
        In the api definition this is a property, but in the python implementation it is a method, because python
        does not support properties with parameters. The parameter is the configuration name.
        """
        return ICustomPropertyManager(self.com_object.CustomPropertyManager(config_name))

    @property
    def display_title(self) -> str:
        """
        Gets the display title of this component.

        Reference:
        https://help.solidworks.com/2024/english/api/sldworksapi/SolidWorks.Interop.sldworks~SolidWorks.Interop.sldworks.IComponent2~DisplayTitle.html
        """
        return self.com_object.DisplayTitle

    @display_title.setter
    def display_title(self, value: str):
        self.com_object.DisplayTitle = value

    @property
    def i_material_property_values(self) -> List[float]:
        """
        Gets the material property values for this component.

        Reference:
        https://help.solidworks.com/2024/english/api/sldworksapi/SolidWorks.Interop.sldworks~SolidWorks.Interop.sldworks.IComponent2~IMaterialPropertyValues.html
        """
        com_object = self.com_object.IMaterialPropertyValues
        return [float(i) for i in com_object]

    @i_material_property_values.setter
    def i_material_property_values(self, values: List[float]) -> None:
        in_values = VARIANT(VT_ARRAY | VT_R8, values)
        self.com_object.IMaterialPropertyValues = in_values

    @property
    def is_graphics_only(self) -> bool:
        """
        Gets whether this component is graphics only.

        Reference:
        https://help.solidworks.com/2024/english/api/sldworksapi/SolidWorks.Interop.sldworks~SolidWorks.Interop.sldworks.IComponent2~IsGraphicsOnly.html
        """
        return bool(self.com_object.IsGraphicsOnly)

    @is_graphics_only.setter
    def is_graphics_only(self, value: bool) -> None:
        in_value = VARIANT(VT_BOOL, value)
        self.com_object.IsGraphicsOnly = in_value

    @property
    def is_speedpak(self) -> bool:
        """
        Gets whether this component is SpeedPak.

        Reference:
        https://help.solidworks.com/2024/english/api/sldworksapi/SolidWorks.Interop.sldworks~SolidWorks.Interop.sldworks.IComponent2~IsSpeedPak.html
        """
        return bool(self.com_object.IsSpeedPak)

    @is_speedpak.setter
    def is_speedpak(self, value: bool) -> None:
        in_value = VARIANT(VT_BOOL, value)
        self.com_object.IsSpeedPak = in_value

    @property
    def is_virtual(self) -> bool:
        """
        Gets whether this component is a virtual component.

        Reference:
        https://help.solidworks.com/2024/english/api/sldworksapi/SolidWorks.Interop.sldworks~SolidWorks.Interop.sldworks.IComponent2~IsVirtual.html
        """
        return bool(self.com_object.IsVirtual)

    @property
    def material_property_values(self) -> List[float] | None:
        """
        Gets the material property values for this component.

        Reference:
        https://help.solidworks.com/2024/english/api/sldworksapi/SolidWorks.Interop.sldworks~SolidWorks.Interop.sldworks.IComponent2~MaterialPropertyValues.html
        """
        com_object = self.com_object.MaterialPropertyValues
        if com_object is None:
            return None
        return [float(i) for i in com_object]

    @material_property_values.setter
    def material_property_values(self, values: List[float]) -> None:
        in_values = VARIANT(VT_ARRAY | VT_R8, values)
        self.com_object.MaterialPropertyValues = in_values

    @property
    def name2(self) -> str:
        """
        Gets the name of this component.

        Reference:
        https://help.solidworks.com/2024/english/api/sldworksapi/SolidWorks.Interop.sldworks~SolidWorks.Interop.sldworks.IComponent2~Name2.html
        """
        return self.com_object.Name2

    @name2.setter
    def name2(self, value: str):
        self.com_object.Name2 = value

    @property
    def presentation_transform(self) -> IMathTransform:
        """
        Gets or sets the component transform.

        Reference:
        https://help.solidworks.com/2024/english/api/sldworksapi/SolidWorks.Interop.sldworks~SolidWorks.Interop.sldworks.IComponent2~PresentationTransform.html
        """
        com_object = self.com_object.PresentationTransform
        return IMathTransform(com_object)

    @presentation_transform.setter
    def presentation_transform(self, value: IMathTransform) -> None:
        in_value = VARIANT(VT_DISPATCH, value.com_object)
        self.com_object.PresentationTransform = in_value

    @property
    def referenced_configuration(self) -> str:
        """
        Gets or sets the active configuration used by this component.

        Reference:
        https://help.solidworks.com/2024/english/api/sldworksapi/SolidWorks.Interop.sldworks~SolidWorks.Interop.sldworks.IComponent2~ReferencedConfiguration.html
        """
        return self.com_object.ReferencedConfiguration

    @referenced_configuration.setter
    def referenced_configuration(self, value: str) -> None:
        self.com_object.referenced_configuration = value

    @property
    def referenced_display_state2(self) -> str:
        """
        Gets or sets the active display state of this component.

        Reference:
        https://help.solidworks.com/2024/english/api/sldworksapi/SolidWorks.Interop.sldworks~SolidWorks.Interop.sldworks.IComponent2~ReferencedDisplayState2.html
        """
        return self.com_object.ReferencedDisplayState2

    @referenced_display_state2.setter
    def referenced_display_state2(self, value: str) -> None:
        self.com_object.ReferencedDisplayState2 = value

    @property
    def solving(self) -> SWComponentSolvingOptionE | int:
        """
        Gets the Solve as option (rigid or flexible) of this component.

        Reference:
        https://help.solidworks.com/2024/english/api/sldworksapi/SolidWorks.Interop.sldworks~SolidWorks.Interop.sldworks.IComponent2~Solving.html
        """
        com_object = self.com_object.Solving
        if isinstance(com_object, int):
            return com_object
        return SWComponentSolvingOptionE(com_object)

    @solving.setter
    def solving(self, value: SWComponentSolvingOptionE) -> None:
        in_value = VARIANT(VT_I4, value.value)
        self.com_object.Solving = in_value

    @property
    def transform2(self) -> IMathTransform:
        """
        Gets or sets the component transform.

        Reference:
        https://help.solidworks.com/2024/english/api/sldworksapi/SolidWorks.Interop.sldworks~SolidWorks.Interop.sldworks.IComponent2~Transform2.html
        """
        com_object = self.com_object.Transform2
        return IMathTransform(com_object)

    @transform2.setter
    def transform2(self, value: IMathTransform) -> None:
        in_value = VARIANT(VT_DISPATCH, value.com_object)
        self.com_object.Transform2 = in_value

    @property
    def use_named_configuration(self) -> bool:
        """
        Gets whether a specified configuration or the in-use/last active configuration is used.

        Reference:
        https://help.solidworks.com/2024/english/api/sldworksapi/SolidWorks.Interop.sldworks~SolidWorks.Interop.sldworks.IComponent2~UseNamedConfiguration.html
        """
        return bool(self.com_object.UseNamedConfiguration)

    @use_named_configuration.setter
    def use_named_configuration(self, value: bool) -> None:
        in_value = VARIANT(VT_BOOL, value)
        self.com_object.UseNamedConfiguration = in_value

    @property
    def visible(self) -> bool:
        """
        Gets or sets the visibility state of this component.

        Reference:
        https://help.solidworks.com/2024/english/api/sldworksapi/SolidWorks.Interop.sldworks~SolidWorks.Interop.sldworks.IComponent2~Visible.html
        """
        return bool(self.com_object.Visible)

    @visible.setter
    def visible(self, value: bool) -> None:
        in_value = VARIANT(VT_BOOL, value)
        self.com_object.Visible = in_value

    def add_property_extension(self, property_extension: int | float | str) -> int:
        """
        Adds a property extension to this component.

        Reference:
        https://help.solidworks.com/2024/english/api/sldworksapi/SolidWorks.Interop.sldworks~SolidWorks.Interop.sldworks.IComponent2~AddPropertyExtension.html
        """
        in_property_extension = VARIANT(VT_I4 | VT_R8 | VT_BSTR, property_extension)
        com_object = self.com_object.AddPropertyExtension(in_property_extension)
        return int(com_object)

    def de_select(self) -> bool:
        """
        Deselects this component.

        Reference:
        https://help.solidworks.com/2024/english/api/sldworksapi/SolidWorks.Interop.sldworks~SolidWorks.Interop.sldworks.IComponent2~DeSelect.html
        """
        return bool(self.com_object.DeSelect)

    def enum_bodies3(self, body_type: SWBodyTypeE, visible_only: bool) -> IEnumBodies2:
        """
        Gets the bodies in the component in a multibody part.

        Reference:
        https://help.solidworks.com/2024/english/api/sldworksapi/SolidWorks.Interop.sldworks~SolidWorks.Interop.sldworks.IComponent2~EnumBodies3.html
        """
        in_body_type = VARIANT(VT_I4, body_type.value)
        in_visible_only = VARIANT(VT_BOOL, visible_only)

        com_object = self.com_object.EnumBodies3(in_body_type, in_visible_only)
        return IEnumBodies2(com_object)

    def get_children(self) -> List["IComponent2"]:
        """
        Gets all of the children components of this component.

        Reference:
        https://help.solidworks.com/2024/english/api/sldworksapi/SolidWorks.Interop.sldworks~SolidWorks.Interop.sldworks.IComponent2~GetChildren.html
        """
        com_object = self.com_object.GetChildren
        return [IComponent2(i) for i in com_object]

    def get_exclude_from_bom2(self, config_opt: SWInConfigurationOptsE, config_names: List[str] | None) -> List[bool]:
        """
        Gets whether this component is excluded from the bills of materials (BOMs) in the specified configurations.

        Reference:
        https://help.solidworks.com/2024/english/api/sldworksapi/SolidWorks.Interop.sldworks~SolidWorks.Interop.sldworks.IComponent2~GetExcludeFromBOM2.html
        """
        in_config_opt = VARIANT(VT_I4, config_opt.value)
        in_config_names = (
            VARIANT(VT_BSTR | VT_ARRAY, [str(i) for i in config_names]) if config_names else VARIANT(VT_NULL, None)
        )

        com_object = self.com_object.GetExcludeFromBOM2(in_config_opt, in_config_names)
        return [bool(i) for i in com_object]

    def get_id(self) -> int:
        """
        Gets the component ID for this component.

        Reference:
        https://help.solidworks.com/2024/english/api/sldworksapi/SolidWorks.Interop.sldworks~SolidWorks.Interop.sldworks.IComponent2~GetID.html
        """
        return int(self.com_object.GetID)

    def get_model_doc2(self) -> "IModelDoc2 | None":
        """
        Gets the model document for this component.

        Reference:
        https://help.solidworks.com/2024/english/api/sldworksapi/SolidWorks.Interop.sldworks~SolidWorks.Interop.sldworks.IComponent2~GetModelDoc2.html
        """
        from pyswx.api.sldworks.interfaces.i_model_doc_2 import IModelDoc2

        com_object = self.com_object.GetModelDoc2
        return IModelDoc2(com_object) if com_object else None

    def get_parent(self) -> "IComponent2 | None":
        """
        Gets the parent component.

        Reference:
        https://help.solidworks.com/2024/english/api/sldworksapi/SolidWorks.Interop.sldworks~SolidWorks.Interop.sldworks.IComponent2~GetParent.html
        """
        com_object = self.com_object.GetParent
        return IComponent2(com_object) if com_object else None

    def get_path_name(self) -> Path:
        """
        Gets the full path name for this component.

        Reference:
        https://help.solidworks.com/2024/english/api/sldworksapi/SolidWorks.Interop.sldworks~SolidWorks.Interop.sldworks.IComponent2~GetPathName.html
        """
        return Path(self.com_object.GetPathName)

    def get_suppression2(self) -> SWComponentSuppressionStateE:
        """
        Gets the suppression state of this component.

        Reference:
        https://help.solidworks.com/2024/english/api/sldworksapi/SolidWorks.Interop.sldworks~SolidWorks.Interop.sldworks.IComponent2~GetSuppression2.html
        """
        return SWComponentSuppressionStateE(self.com_object.GetSuppression2)

    def get_type(self) -> SWDocumentTypesE:
        """
        Gets this lightweight assembly component's document type.

        Reference:
        https://help.solidworks.com/2024/english/api/sldworksapi/SolidWorks.Interop.sldworks~SolidWorks.Interop.sldworks.IComponent2~GetType.html
        """
        return SWDocumentTypesE(self.com_object.GetType)

    def is_envelope(self) -> bool:
        """
        Gets whether this component is an envelope.

        Reference:
        https://help.solidworks.com/2024/english/api/sldworksapi/solidworks.interop.sldworks~solidworks.interop.sldworks.icomponent2~isenvelope.html
        """
        return bool(self.com_object.IsEnvelope)

    def is_hidden(self, consider_suppressed: bool) -> bool:
        """
        Gets whether this component is hidden or suppressed.

        Reference:
        https://help.solidworks.com/2024/english/api/sldworksapi/solidworks.interop.sldworks~solidworks.interop.sldworks.icomponent2~ishidden.html
        """
        in_consider_suppressed = VARIANT(VT_BOOL, consider_suppressed)
        return bool(self.com_object.IsHidden(in_consider_suppressed))



"""
ICustomPropertyManager Interface Members

Reference:
https://help.solidworks.com/2024/english/api/sldworksapi/SolidWorks.Interop.sldworks~SolidWorks.Interop.sldworks.ICustomPropertyManager_members.html

Status: 🟢
"""

from typing import List
from typing import Tuple

from pythoncom import VT_BOOL
from pythoncom import VT_BSTR
from pythoncom import VT_BYREF
from pythoncom import VT_I4
from pythoncom import VT_VARIANT
from win32com.client import VARIANT

from pyswx.api.base_interface import BaseInterface
from pyswx.api.swconst.enumerations import SWCustomInfoAddResultE
from pyswx.api.swconst.enumerations import SWCustomInfoDeleteResultE
from pyswx.api.swconst.enumerations import SWCustomInfoGetResultE
from pyswx.api.swconst.enumerations import SWCustomInfoSetResultE
from pyswx.api.swconst.enumerations import SWCustomInfoTypeE
from pyswx.api.swconst.enumerations import SWCustomLinkSetResultE
from pyswx.api.swconst.enumerations import SWCustomPropertyAddOptionE

type ValueOut = str
type ResolvedValueOut = str
type WasResolvedOut = bool
type LinkToPropertyOut = bool

type NumberOfProperties = int
type Count = int
type PropNames = List[str]
type PropTypes = List[SWCustomInfoTypeE]
type PropValues = List[str]
type Resolved = List[SWCustomInfoGetResultE]
type PropLink = List[int]


class ICustomPropertyManager(BaseInterface):
    def __init__(self, com_object):
        super().__init__()
        self.com_object = com_object

    def __repr__(self) -> str:
        return f"ICustomPropertyManager({self.com_object})"

    @property
    def count(self) -> int:
        """
        Gets the number of custom properties.

        Reference:
        https://help.solidworks.com/2024/english/api/sldworksapi/SolidWorks.Interop.sldworks~SolidWorks.Interop.sldworks.ICustomPropertyManager~Count.html
        """
        return int(self.com_object.Count)

    @property
    def link_all(self) -> bool:
        """
        Gets or sets whether to link or unlink all custom properties to or from their parent part.

        Reference:
        https://help.solidworks.com/2024/english/api/sldworksapi/SolidWorks.Interop.sldworks~SolidWorks.Interop.sldworks.ICustomPropertyManager~LinkAll.html
        """
        return bool(self.com_object.LinkAll)

    @link_all.setter
    def link_all(self, value: bool):
        self.com_object.LinkAll = value

    @property
    def owner(self) -> str:
        """
                Gets the owner of this custom property.

                Reference:
        https://help.solidworks.com/2024/english/api/sldworksapi/SolidWorks.Interop.sldworks~SolidWorks.Interop.sldworks.ICustomPropertyManager~Owner.html
        """
        return str(self.com_object.Owner)

    def add3(
        self,
        field_name: str,
        field_type: SWCustomInfoTypeE,
        field_value: str,
        overwrite_existing: SWCustomPropertyAddOptionE,
    ) -> SWCustomInfoAddResultE:
        """
        Adds a custom property to a configuration or model document.

        Reference:
        https://help.solidworks.com/2024/english/api/sldworksapi/SolidWorks.Interop.sldworks~SolidWorks.Interop.sldworks.ICustomPropertyManager~Add3.html
        """
        return SWCustomInfoAddResultE(
            self.com_object.Add3(field_name, field_type.value, field_value, overwrite_existing.value)
        )

    def delete2(self, field_name: str) -> SWCustomInfoDeleteResultE:
        """
        Deletes the specified custom property.

        Reference:
        https://help.solidworks.com/2024/english/api/sldworksapi/SolidWorks.Interop.sldworks~SolidWorks.Interop.sldworks.ICustomPropertyManager~Delete2.html
        """
        return SWCustomInfoDeleteResultE(self.com_object.Delete2(field_name))

    def get6(
        self, field_name: str, use_cached: bool
    ) -> Tuple[SWCustomInfoGetResultE, ValueOut, ResolvedValueOut, WasResolvedOut, LinkToPropertyOut]:
        """
        Gets the value and the evaluated value of the specified custom property.

        Reference:
        https://help.solidworks.com/2024/english/api/sldworksapi/SolidWorks.Interop.sldworks~SolidWorks.Interop.sldworks.ICustomPropertyManager~Get6.html
        """

        in_field_name = VARIANT(VT_BSTR, field_name)
        in_use_cached = VARIANT(VT_I4, int(use_cached))

        out_val_out = VARIANT(VT_BYREF | VT_BSTR, None)
        out_resolved_val_out = VARIANT(VT_BYREF | VT_BSTR, None)
        out_was_resolved = VARIANT(VT_BYREF | VT_BOOL, None)
        out_link_to_property = VARIANT(VT_BYREF | VT_BOOL, None)

        com_object = self.com_object.Get6(
            in_field_name,
            in_use_cached,
            out_val_out,
            out_resolved_val_out,
            out_was_resolved,
            out_link_to_property,
        )

        return (
            SWCustomInfoGetResultE(com_object),
            str(out_val_out.value),
            str(out_resolved_val_out.value),
            bool(out_was_resolved.value),
            bool(out_link_to_property.value),
        )

    def get_all3(self) -> Tuple[NumberOfProperties, PropNames, PropTypes, PropValues, Resolved, PropLink]:
        """
        Gets all of the custom properties for this configuration.

        Reference:
        https://help.solidworks.com/2024/english/api/sldworksapi/SolidWorks.Interop.sldworks~SolidWorks.Interop.sldworks.ICustomPropertyManager~GetAll3.html
        """
        out_prop_names = VARIANT(VT_VARIANT | VT_BYREF, [])
        out_prop_types = VARIANT(VT_VARIANT | VT_BYREF, [])
        out_prop_values = VARIANT(VT_VARIANT | VT_BYREF, [])
        out_prop_resolved = VARIANT(VT_VARIANT | VT_BYREF, [])
        out_prop_link = VARIANT(VT_VARIANT | VT_BYREF, [])

        com_object = self.com_object.GetAll3(
            out_prop_names,
            out_prop_types,
            out_prop_values,
            out_prop_resolved,
            out_prop_link,
        )

        return (
            int(com_object),
            [str(i) for i in out_prop_names.value],
            [SWCustomInfoTypeE(i) for i in out_prop_types.value],
            [str(i) for i in out_prop_values.value],
            [SWCustomInfoGetResultE(i) for i in out_prop_resolved.value],
            [int(i) for i in out_prop_link.value],
        )

    def get_names(self) -> List[str]:
        """
        Gets the names of the custom properties.

        Reference:
        https://help.solidworks.com/2024/english/api/sldworksapi/SolidWorks.Interop.sldworks~SolidWorks.Interop.sldworks.ICustomPropertyManager~GetNames.html
        """
        com_object = self.com_object.GetNames
        return [str(i) for i in com_object] if com_object else []

    def get_type2(self, field_name: str) -> SWCustomInfoTypeE:
        """
        Gets the type of the specified custom property for a configuration or model document.

        Reference:
        https://help.solidworks.com/2024/english/api/sldworksapi/SolidWorks.Interop.sldworks~SolidWorks.Interop.sldworks.ICustomPropertyManager~GetType2.html
        """
        return SWCustomInfoTypeE(self.com_object.GetType2(field_name))

    def i_get_all(self) -> Tuple[Count, PropNames, PropTypes, PropValues]:
        """
        Gets all of the custom properties for this configuration.

        Reference:
        https://help.solidworks.com/2024/english/api/sldworksapi/SolidWorks.Interop.sldworks~SolidWorks.Interop.sldworks.ICustomPropertyManager~IGetAll.html
        """

        out_count = VARIANT(VT_BYREF | VT_I4, None)
        out_prop_names = VARIANT(VT_VARIANT | VT_BYREF, [])
        out_prop_types = VARIANT(VT_VARIANT | VT_BYREF, [])
        out_prop_values = VARIANT(VT_VARIANT | VT_BYREF, [])

        com_object = self.com_object.IGetAll(out_count, out_prop_names, out_prop_types, out_prop_values)

        return (
            int(out_count.value),
            [str(i) for i in out_prop_names.value],
            [SWCustomInfoTypeE(i) for i in out_prop_types.value],
            [str(i) for i in out_prop_values.value],
        )

    def i_get_names(self) -> Count:
        """
        Gets the names of the custom properties.

        Reference:
        https://help.solidworks.com/2024/english/api/sldworksapi/SolidWorks.Interop.sldworks~SolidWorks.Interop.sldworks.ICustomPropertyManager~IGetNames.html
        """

        out_count = VARIANT(VT_BYREF | VT_I4, None)

        com_object = self.com_object.IGetNames(out_count)

        return int(out_count.value)

    def is_custom_property_editable(self, property_name: str, configuration_name: str) -> bool:
        """
        Gets whether the specified custom property is editable in the specified configuration.

        Reference:
        https://help.solidworks.com/2024/english/api/sldworksapi/SolidWorks.Interop.sldworks~SolidWorks.Interop.sldworks.ICustomPropertyManager~IsCustomPropertyEditable.html
        """

        in_property_name = VARIANT(VT_BSTR, property_name)
        in_configuration_name = VARIANT(VT_BSTR, configuration_name)

        com_object = self.com_object.IsCustomPropertyEditable(in_property_name, in_configuration_name)

        return bool(com_object)

    def link_property(self, field_name: str, field_link: bool) -> SWCustomLinkSetResultE:
        """
        Sets whether to link or unlink the specified custom property to or from its parent part.

        Reference:
        https://help.solidworks.com/2024/english/api/sldworksapi/SolidWorks.Interop.sldworks~SolidWorks.Interop.sldworks.ICustomPropertyManager~LinkProperty.html
        """
        return SWCustomLinkSetResultE(self.com_object.LinkProperty(field_name, field_link))

    def set2(self, field_name: str, field_value: str) -> SWCustomInfoSetResultE:
        """
        Sets the value of the specified custom property.

        Reference:
        https://help.solidworks.com/2024/english/api/sldworksapi/SolidWorks.Interop.sldworks~SolidWorks.Interop.sldworks.ICustomPropertyManager~Set2.html
        """
        return SWCustomInfoSetResultE(self.com_object.Set2(field_name, field_value))



"""
swComponentSolvingOption_e Enumeration

Reference:
https://help.solidworks.com/2024/english/api/swconst/SOLIDWORKS.Interop.swconst~SOLIDWORKS.Interop.swconst.swComponentSolvingOption_e.html

Status: 🟢
"""

from enum import IntEnum


class SWComponentSolvingOptionE(IntEnum):
    """Specifies options for resolving components in assemblies."""

    SW_COMPONENT_RIGID_SOLVING = 0  # Component is solved as rigid (default)
    SW_COMPONENT_FLEXIBLE_SOLVING = 1  # Component is solved as flexible



"""
swCADFamilyCfgOptions_e Enumeration

Reference:
https://help.solidworks.com/2024/english/api/swconst/SOLIDWORKS.Interop.swconst~SOLIDWORKS.Interop.swconst.swCADFamilyCfgOptions_e.html

Status: 🟢
"""

from enum import IntFlag


class SWCADFamilyCfgOptionsE(IntFlag):
    """SOLIDWORKS CONNECTED family configuration options. Bitmask."""

    SW_CAD_FAMILY_CFG_OPTION_SUPPRESS_NEW_FEATURES = 1  # Suppress new features
    SW_CAD_FAMILY_CFG_OPTION_SUPPRESS_NEW_COMPONENTS = 2  # Suppress new components
    SW_CAD_FAMILY_CFG_OPTION_DONT_ACTIVATE = 4  # Do not activate



"""
swSaveAsVersion_e Enumeration

Reference:
https://help.solidworks.com/2024/english/api/swconst/SOLIDWORKS.Interop.swconst~SOLIDWORKS.Interop.swconst.swSaveAsVersion_e.html

Status: 🟢
"""

from enum import IntEnum


class SWSaveAsVersionE(IntEnum):
    SW_SAVE_AS_CURRENT_VERSION = 0
    SW_SAVE_AS_FORMAT_PRO_E = 2
    SW_SAVE_AS_STANDARD_DRAWING = 3
    SW_SAVE_AS_DETACHED_DRAWING = 4



"""
swConfigurationOptions2_e Enumeration

Reference:
https://help.solidworks.com/2024/english/api/swconst/SOLIDWORKS.Interop.swconst~SOLIDWORKS.Interop.swconst.swConfigurationOptions2_e.html

Status: 🟢
"""

from enum import IntEnum


class SWConfigurationOptions2E(IntEnum):
    """Option bits used when setting configuration options. Bitmask."""

    SW_CONFIG_OPTION_USE_ALTERNATE_NAME = 1  # Use an alternate configuration name
    SW_CONFIG_OPTION_DONT_SHOW_PARTS_IN_BOM = (
        2  # Show sub-assemblies instead of child components in BOM
    )
    SW_CONFIG_OPTION_SUPPRESS_BY_DEFAULT = 4  # Suppress newly added features and mates
    SW_CONFIG_OPTION_HIDE_BY_DEFAULT = 8  # Hide newly added components
    SW_CONFIG_OPTION_MIN_FEATURE_MANAGER = 16  # Suppress new components
    SW_CONFIG_OPTION_LINK_TO_PARENT = 64  # Link component to parent configuration
    SW_CONFIG_OPTION_DONT_ACTIVATE = 128  # Do not activate the configuration
    SW_CONFIG_OPTION_DO_DISSOLVE_IN_BOM = (
        256  # Dissolve configuration in BOM and promote child components
    )
    SW_CONFIG_OPTION_USE_DESCRIPTION_IN_BOM = 512  # Use description in BOM



# Filename: sw_select_type_e.py

"""
swSelectType_e Enumeration

Values for types of returned IDs.

Reference:
https://help.solidworks.com/2024/english/api/swconst/SOLIDWORKS.Interop.swconst~SOLIDWORKS.Interop.swconst.swSelectType_e.html

Status: 🟢
"""

from enum import IntEnum


class SWSelectTypeE(IntEnum):
    """Enumeration of selection type values (returned IDs)."""

    SW_SEL_ADVSTRUCTMEMBER = 295  # See Remarks
    SW_SEL_ANNOTATIONTABLES = 98  # See Remarks
    SW_SEL_ANNOTATIONVIEW = 139  # See Remarks
    SW_SEL_ARROWS = 49  # See Remarks
    SW_SEL_ATTRIBUTES = 8  # See Remarks
    SW_SEL_BELTCHAINFEATS = 149  # See Remarks
    SW_SEL_BLOCKDEF = 99  # See Remarks
    SW_SEL_BLOCKINST = 93  # See Remarks
    SW_SEL_BODYFEATURES = 22  # See Remarks
    SW_SEL_BODYFOLDER = 118  # See Remarks
    SW_SEL_BOMFEATURES = 97  # See Remarks
    SW_SEL_BOMS = 54  # See Remarks
    SW_SEL_BOMTEMPS = 64  # See Remarks
    SW_SEL_BORDER = 254  # See Remarks
    SW_SEL_BREAKLINES = 31  # See Remarks
    SW_SEL_BROWSERITEM = 69  # See Remarks
    SW_SEL_CAMERAS = 136  # See Remarks
    SW_SEL_CENTERLINES = 103  # See Remarks
    SW_SEL_CENTERMARKS = 28  # See Remarks
    SW_SEL_CENTERMARKSYMS = 100  # See Remarks
    SW_SEL_COMMENT = 127  # See Remarks
    SW_SEL_COMMENTSFOLDER = 126  # See Remarks
    SW_SEL_COMPONENTS = 20  # See Remarks
    SW_SEL_COMPPATTERN = 37  # See Remarks
    SW_SEL_COMPSDONTOVERRIDE = 72  # See Remarks
    SW_SEL_CONFIGURATIONS = 47  # See Remarks
    SW_SEL_CONNECTIONPOINTS = 66  # See Remarks
    SW_SEL_COORDSYS = 61  # See Remarks
    SW_SEL_COSMETICWELDS = 220  # See Remarks
    SW_SEL_CTHREADS = 39  # See Remarks
    SW_SEL_CUSTOMSYMBOLS = 60  # See Remarks
    SW_SEL_DATUMAXES = 5  # See Remarks
    SW_SEL_DATUMLINES = 62  # See Remarks
    SW_SEL_DATUMPLANES = 4  # See Remarks
    SW_SEL_DATUMPOINTS = 6  # See Remarks
    SW_SEL_DATUMTAGS = 36  # See Remarks
    SW_SEL_DCABINETS = 42  # See Remarks
    SW_SEL_DETAILCIRCLES = 17  # See Remarks
    SW_SEL_DIMENSIONS = 14  # See Remarks
    SW_SEL_DISPLAYSTATE = 148  # See Remarks
    SW_SEL_DOCSFOLDER = 125  # See Remarks
    SW_SEL_DOWELSYMS = 86  # See Remarks
    SW_SEL_DRAWINGVIEWS = 12  # See Remarks
    SW_SEL_DTMTARGS = 40  # See Remarks
    SW_SEL_EDGES = 1  # See Remarks
    SW_SEL_EMBEDLINKDOC = 123  # See Remarks
    SW_SEL_EMPTYSPACE = 72  # See Remarks (duplicate of SW_SEL_COMPSDONTOVERRIDE)
    SW_SEL_EQNFOLDER = 55  # See Remarks
    SW_SEL_EVERYTHING = -3  # See Remarks
    SW_SEL_EXCLUDEMANIPULATORS = 111  # See Remarks
    SW_SEL_EXPLLINES = 45  # See Remarks
    SW_SEL_EXPLSTEPS = 44  # See Remarks
    SW_SEL_EXPLVIEWS = 43  # See Remarks
    SW_SEL_EXTSKETCHPOINTS = 25  # See Remarks
    SW_SEL_EXTSKETCHSEGS = 24  # See Remarks
    SW_SEL_EXTSKETCHTEXT = 88  # See Remarks
    SW_SEL_FABRICATEDROUTE = 70  # See Remarks
    SW_SEL_FACES = 2  # See Remarks
    SW_SEL_FACETS = 268  # See Remarks
    SW_SEL_FRAMEPOINT = 77  # See Remarks
    SW_SEL_FTRFOLDER = 94  # See Remarks
    SW_SEL_GENERALTABLEFEAT = 142  # See Remarks
    SW_SEL_GRAPHICSBODY = 262  # See Remarks
    SW_SEL_GTOLS = 13  # See Remarks
    SW_SEL_HELIX = 26  # See Remarks
    SW_SEL_HOLESERIES = 83  # See Remarks
    SW_SEL_HOLETABLEAXES = 105  # See Remarks
    SW_SEL_HOLETABLEFEATS = 104  # See Remarks
    SW_SEL_IMPORTFOLDER = 57  # See Remarks
    SW_SEL_INCONTEXTFEAT = 29  # See Remarks
    SW_SEL_INCONTEXTFEATS = 32  # See Remarks
    SW_SEL_JOURNAL = 124  # See Remarks
    SW_SEL_LEADERS = 84  # See Remarks
    SW_SEL_LIGHTS = 73  # See Remarks
    SW_SEL_LOCATIONS = -2  # See Remarks
    SW_SEL_MAGNETICLINES = 225  # See Remarks
    SW_SEL_MANIPULATORS = 79  # See Remarks
    SW_SEL_MATEGROUP = 30  # See Remarks
    SW_SEL_MATEGROUPS = 33  # See Remarks
    SW_SEL_MATES = 21  # See Remarks
    SW_SEL_MATESUPPLEMENT = 138  # See Remarks
    SW_SEL_MESHFACETEDGES = 269  # See Remarks
    SW_SEL_MESHFACETVERTICES = 270  # See Remarks
    SW_SEL_MESHSOLIDBODIES = 274  # See Remarks
    SW_SEL_MIDPOINTS = 59  # See Remarks
    SW_SEL_NOTES = 15  # See Remarks
    SW_SEL_NOTHING = 0  # See Remarks
    SW_SEL_OBJGROUP = 155  # See Remarks
    SW_SEL_OBJHANDLES = 48  # See Remarks
    SW_SEL_OLEITEMS = 7  # See Remarks
    SW_SEL_PICTUREBODIES = 80  # See Remarks
    SW_SEL_PLANESECTIONS = 219  # See Remarks
    SW_SEL_POINTREFS = 41  # See Remarks
    SW_SEL_POSGROUP = 68  # See Remarks
    SW_SEL_PUNCHTABLEFEATS = 234  # See Remarks
    SW_SEL_REFCURVES = 23  # See Remarks
    SW_SEL_REFEDGES = 51  # See Remarks
    SW_SEL_REFERENCECURVES = 26  # See Remarks (duplicate of SW_SEL_HELIX)
    SW_SEL_REFFACES = 52  # See Remarks
    SW_SEL_REFSILHOUETTE = 53  # See Remarks
    SW_SEL_REFSURFACES = 27  # See Remarks
    SW_SEL_REVISIONCLOUDS = 240  # See Remarks
    SW_SEL_REVISIONTABLE = 113  # See Remarks
    SW_SEL_REVISIONTABLEFEAT = 119  # See Remarks
    SW_SEL_ROUTECURVES = 63  # See Remarks
    SW_SEL_ROUTEPOINTS = 65  # See Remarks
    SW_SEL_ROUTESWEEPS = 67  # See Remarks
    SW_SEL_SECTIONLINES = 16  # See Remarks
    SW_SEL_SECTIONTEXT = 18  # See Remarks
    SW_SEL_SELECTIONSETFOLDER = 258  # See Remarks
    SW_SEL_SELECTIONSETNODE = 259  # See Remarks
    SW_SEL_SFSYMBOLS = 35  # See Remarks
    SW_SEL_SHEETS = 19  # See Remarks
    SW_SEL_SILHOUETTES = 46  # See Remarks
    SW_SEL_SIMELEMENT = 102  # See Remarks
    SW_SEL_SIMULATION = 101  # See Remarks
    SW_SEL_SKETCHBITMAP = 85  # See Remarks
    SW_SEL_SKETCHCONTOUR = 96  # See Remarks
    SW_SEL_SKETCHES = 9  # See Remarks
    SW_SEL_SKETCHHATCH = 56  # See Remarks
    SW_SEL_SKETCHPOINTFEAT = 71  # See Remarks
    SW_SEL_SKETCHPOINTS = 11  # See Remarks
    SW_SEL_SKETCHREGION = 95  # See Remarks
    SW_SEL_SKETCHSEGS = 10  # See Remarks
    SW_SEL_SKETCHTEXT = 34  # See Remarks
    SW_SEL_SOLIDBODIES = 76  # See Remarks
    SW_SEL_SOLIDBODIESFIRST = 81  # See Remarks
    SW_SEL_SUBATOMFOLDER = 121  # See Remarks
    SW_SEL_SUBSKETCHDEF = 154  # See Remarks
    SW_SEL_SUBSKETCHINST = 114  # See Remarks
    SW_SEL_SUBWELDFOLDER = 107  # See Remarks
    SW_SEL_SURFACEBODIES = 75  # See Remarks
    SW_SEL_SURFBODIESFIRST = 78  # See Remarks
    SW_SEL_SWIFTANNOTATIONS = 130  # See Remarks
    SW_SEL_SWIFTFEATURES = 132  # See Remarks
    SW_SEL_SWIFTSCHEMA = 159  # See Remarks
    SW_SEL_TITLEBLOCK = 192  # See Remarks
    SW_SEL_TITLEBLOCKTABLEFEAT = 206  # See Remarks
    SW_SEL_UNSUPPORTED = -1  # See Remarks
    SW_SEL_VERTICES = 3  # See Remarks
    SW_SEL_VIEWERHYPERLINK = 58  # See Remarks
    SW_SEL_WELDBEADS = 122  # See Remarks
    SW_SEL_WELDMENT = 106  # See Remarks
    SW_SEL_WELDMENTTABLEFEATS = 116  # See Remarks
    SW_SEL_WELDS = 38  # See Remarks
    SW_SEL_WIREBODIES = 74  # See Remarks
    SW_SEL_ZONES = 50  # See Remarks



"""
IConfiguration Interface Members

Reference:
https://help.solidworks.com/2024/english/api/sldworksapi/SOLIDWORKS.Interop.sldworks~SOLIDWORKS.Interop.sldworks.IConfiguration.html

Status: 🔴
"""

from pythoncom import VT_BOOL
from pythoncom import VT_BSTR
from pythoncom import VT_BYREF
from pythoncom import VT_I4
from pythoncom import VT_R8
from win32com.client import VARIANT

from pyswx.api.base_interface import BaseInterface
from pyswx.api.sldworks.interfaces.i_component_2 import IComponent2
from pyswx.api.sldworks.interfaces.i_custom_property_manager import ICustomPropertyManager
from pyswx.api.sldworks.interfaces.i_dim_xpert_manager import IDimXpertManager
from pyswx.api.sldworks.interfaces.i_explode_step import IExplodeStep
from pyswx.api.swconst.enumerations import SWChildComponentInBOMOptionE
from pyswx.api.swconst.enumerations import SWConfigurationTypeE
from pyswx.api.swconst.enumerations import SWCreateExplodeStepErrorE
from pyswx.exceptions import DocumentError


class IConfiguration(BaseInterface):
    def __init__(self, com_object):
        super().__init__()
        self.com_object = com_object

    def __repr__(self) -> str:
        return f"IConfiguration({self.com_object})"

    @property
    def add_rebuild_save_mark(self) -> bool:
        """
        Adds or removes the mark indicating whether the configuration needs to be rebuilt and its configuration data saved every time you save the model document.

        Reference:
        https://help.solidworks.com/2024/english/api/sldworksapi/SolidWorks.Interop.sldworks~SolidWorks.Interop.sldworks.IConfiguration~AddRebuildSaveMark.html
        """
        return bool(self.com_object.AddRebuildSaveMark)

    @add_rebuild_save_mark.setter
    def add_rebuild_save_mark(self, value: bool) -> None:
        self.com_object.AddRebuildSaveMark = value

    @property
    def alternate_name(self) -> str:
        """
        Gets or sets the configuration's alternate name (i.e., user-specified name).

        Reference:
        https://help.solidworks.com/2024/english/api/sldworksapi/SolidWorks.Interop.sldworks~SolidWorks.Interop.sldworks.IConfiguration~AlternateName.html
        """
        return str(self.com_object.AlternateName)

    @alternate_name.setter
    def alternate_name(self, value: str) -> None:
        self.com_object.AlternateName = value

    @property
    def bom_part_no_source(self) -> int:
        """
        Gets or sets the source of the part number used in the BOM table.

        Reference:
        https://help.solidworks.com/2024/english/api/sldworksapi/SolidWorks.Interop.sldworks~SolidWorks.Interop.sldworks.IConfiguration~BOMPartNoSource.html
        """
        return int(self.com_object.BOMPartNoSource)

    @bom_part_no_source.setter
    def bom_part_no_source(self, value: int) -> None:
        self.com_object.BOMPartNoSource = value

    @property
    def child_component_display_in_bom(self) -> SWChildComponentInBOMOptionE:
        """
        Gets or sets the child component display option of a configuration in a Bill of Materials (BOM) for an assembly document.

        Reference:
        https://help.solidworks.com/2024/English/api/sldworksapi/SolidWorks.Interop.sldworks~SolidWorks.Interop.sldworks.IConfiguration~ChildComponentDisplayInBOM.html
        """
        return SWChildComponentInBOMOptionE(self.com_object.ChildComponentDisplayInBOM)

    @child_component_display_in_bom.setter
    def child_component_display_in_bom(self, value: SWChildComponentInBOMOptionE) -> None:
        self.com_object.ChildComponentDisplayInBOM = value.value

    @property
    def comment(self) -> str:
        """
        Gets or sets the configuration comment.

        Reference:
        https://help.solidworks.com/2024/english/api/sldworksapi/SolidWorks.Interop.sldworks~SolidWorks.Interop.sldworks.IConfiguration~Comment.html
        """
        return str(self.com_object.Comment)

    @comment.setter
    def comment(self, value: str) -> None:
        self.com_object.Comment = value

    @property
    def custom_property_manager(self) -> ICustomPropertyManager:
        """
        Gets the custom property information for this configuration.

        Reference:
        https://help.solidworks.com/2024/english/api/sldworksapi/SolidWorks.Interop.sldworks~SolidWorks.Interop.sldworks.IConfiguration~CustomPropertyManager.html
        """
        return ICustomPropertyManager(self.com_object.CustomPropertyManager)

    @property
    def description(self) -> str:
        """
        Gets or sets the description of the configuration.

        Reference:
        https://help.solidworks.com/2024/english/api/sldworksapi/SolidWorks.Interop.sldworks~SolidWorks.Interop.sldworks.IConfiguration~Description.html
        """
        return str(self.com_object.Description)

    @description.setter
    def description(self, value: str) -> None:
        self.com_object.Description = value

    def dim_xpert_manager(self, create_schema: bool) -> IDimXpertManager:
        """
        Gets the DimXpert schema for this configuration.

        Reference:
        https://help.solidworks.com/2024/english/api/sldworksapi/SolidWorks.Interop.sldworks~SolidWorks.Interop.sldworks.IConfiguration~DimXpertManager.html

        Note:
        In the api definition this is a property, but in the python implementation it is a method, because python
        does not support properties with parameters. The parameter is the configuration name.
        """
        in_create_schema = VARIANT(VT_BOOL, bool(create_schema))
        return IDimXpertManager(self.com_object.DimXpertManager(in_create_schema))

    @property
    def hide_new_component_models(self) -> bool:
        """
        Gets or sets whether new components are hidden in this inactive configuration.

        Reference:
        https://help.solidworks.com/2024/english/api/sldworksapi/SolidWorks.Interop.sldworks~SolidWorks.Interop.sldworks.IConfiguration~HideNewComponentModels.html
        """
        return bool(self.com_object.HideNewComponentModels)

    @hide_new_component_models.setter
    def hide_new_component_models(self, value: bool) -> None:
        self.com_object.HideNewComponentModels = value

    @property
    def large_design_review_mark(self) -> bool:
        """
        Gets or sets whether to add display data to this configuration when the document is saved.

        Reference:
        https://help.solidworks.com/2024/english/api/sldworksapi/SolidWorks.Interop.sldworks~SolidWorks.Interop.sldworks.IConfiguration~LargeDesignReviewMark.html
        """
        return bool(self.com_object.LargeDesignReviewMark)

    @large_design_review_mark.setter
    def large_design_review_mark(self, value: bool) -> None:
        self.com_object.LargeDesignReviewMark = value

    @property
    def lock(self) -> bool:
        """
        Gets or sets whether the configuration is locked.

        Reference:
        https://help.solidworks.com/2024/english/api/sldworksapi/SolidWorks.Interop.sldworks~SolidWorks.Interop.sldworks.IConfiguration~Lock.html
        """
        return bool(self.com_object.Lock)

    @lock.setter
    def lock(self, value: bool) -> None:
        self.com_object.Lock = value

    @property
    def name(self) -> str:
        """
        Gets or sets the configuration name.

        Reference:
        https://help.solidworks.com/2021/English/api/sldworksapi/SolidWorks.Interop.sldworks~SolidWorks.Interop.sldworks.IConfiguration~Name.html
        """
        return str(self.com_object.Name)

    @name.setter
    def name(self, value: str) -> None:
        self.com_object.Name = value

    @property
    def needs_rebuild(self) -> bool:
        """
        Gets whether the configuration needs to be rebuilt.

        Reference:
        https://help.solidworks.com/2024/english/api/sldworksapi/SolidWorks.Interop.sldworks~SolidWorks.Interop.sldworks.IConfiguration~NeedsRebuild.html
        """
        return bool(self.com_object.NeedsRebuild)

    @needs_rebuild.setter
    def needs_rebuild(self, value: bool) -> None:
        self.com_object.NeedsRebuild = value

    @property
    def representation_shared(self) -> bool:
        """
        Gets or sets whether this SOLIDWORKS Connected Representation configuration is shared.

        Reference:
        https://help.solidworks.com/2024/english/api/sldworksapi/SolidWorks.Interop.sldworks~SolidWorks.Interop.sldworks.IConfiguration~RepresentationShared.html
        """
        return bool(self.com_object.RepresentationShared)

    @representation_shared.setter
    def representation_shared(self, value: bool) -> None:
        self.com_object.RepresentationShared = value

    @property
    def suppress_new_component_models(self) -> bool:
        """
        Gets or sets whether new components are suppressed in this configuration.

        Reference:
        https://help.solidworks.com/2024/english/api/sldworksapi/SolidWorks.Interop.sldworks~SolidWorks.Interop.sldworks.IConfiguration~SuppressNewComponentModels.html
        """
        return bool(self.com_object.SuppressNewComponentModels)

    @suppress_new_component_models.setter
    def suppress_new_component_models(self, value: bool) -> None:
        self.com_object.SuppressNewComponentModels = value

    @property
    def suppress_new_features(self) -> bool:
        """
        Gets or sets whether to suppress new features in this configuration.

        Reference:
        https://help.solidworks.com/2024/english/api/sldworksapi/SolidWorks.Interop.sldworks~SolidWorks.Interop.sldworks.IConfiguration~SuppressNewFeatures.html
        """
        return bool(self.com_object.SuppressNewFeatures)

    @suppress_new_features.setter
    def suppress_new_features(self, value: bool) -> None:
        self.com_object.SuppressNewFeatures = value

    @property
    def type(self) -> SWConfigurationTypeE:
        """
        Gets the type of configuration.

        Reference:
        https://help.solidworks.com/2024/english/api/sldworksapi/SolidWorks.Interop.sldworks~SolidWorks.Interop.sldworks.IConfiguration~Type.html
        """
        return SWConfigurationTypeE(self.com_object.Type)

    @type.setter
    def type(self, value: SWConfigurationTypeE) -> None:
        self.com_object.Type = value.value

    @property
    def use_alternate_name_in_bom(self) -> bool:
        """
        Gets or sets whether the alternate name (i.e., user-specified name) is displayed in the bill of materials for this configuration.

        Reference:
        https://help.solidworks.com/2024/english/api/sldworksapi/SolidWorks.Interop.sldworks~SolidWorks.Interop.sldworks.IConfiguration~UseAlternateNameInBOM.html
        """
        return bool(self.com_object.UseAlternateNameInBOM)

    @use_alternate_name_in_bom.setter
    def use_alternate_name_in_bom(self, value: bool) -> None:
        self.com_object.UseAlternateNameInBOM = value

    @property
    def use_description_in_bom(self) -> bool:
        """
        Gets or sets whether the description of the configuration is displayed in the bill of materials.

        Reference:
        https://help.solidworks.com/2024/english/api/sldworksapi/SolidWorks.Interop.sldworks~SolidWorks.Interop.sldworks.IConfiguration~UseDescriptionInBOM.html
        """
        return bool(self.com_object.UseDescriptionInBOM)

    @use_description_in_bom.setter
    def use_description_in_bom(self, value: bool) -> None:
        self.com_object.UseDescriptionInBOM = value

    def add_explode_step_2(
        self,
        expl_dist: float,
        expl_dir_index: int,
        reverse_dir: bool,
        expl_ang: float,
        rot_axis_index: int,
        reverse_ang: bool,
        rotate_about_origin: bool,
        auto_space_components_on_drag: bool,
    ) -> IExplodeStep:
        """
        Adds a regular (translate and rotate) explode step to the explode view of the active configuration.

        Reference:
        https://help.solidworks.com/2024/english/api/sldworksapi/SolidWorks.Interop.sldworks~SolidWorks.Interop.sldworks.IConfiguration~AddExplodeStep2.html

        Raises:
            DocumentError: Raised if there is an error.
        """
        in_expl_dist = VARIANT(VT_R8, expl_dist)
        in_expl_dir_index = VARIANT(VT_I4, expl_dir_index)
        in_reverse_dir = VARIANT(VT_BOOL, reverse_dir)
        in_expl_ang = VARIANT(VT_R8, expl_ang)
        in_rot_axis_index = VARIANT(VT_I4, rot_axis_index)
        in_reverse_ang = VARIANT(VT_BOOL, reverse_ang)
        in_rotate_about_origin = VARIANT(VT_BOOL, rotate_about_origin)
        in_auto_space_components_on_drag = VARIANT(VT_BOOL, auto_space_components_on_drag)

        out_errors = VARIANT(VT_BYREF | VT_I4, None)

        com_object = self.com_object.AddExplodeStep2(
            in_expl_dist,
            in_expl_dir_index,
            in_reverse_dir,
            in_expl_ang,
            in_rot_axis_index,
            in_reverse_ang,
            in_rotate_about_origin,
            in_auto_space_components_on_drag,
            out_errors,
        )
        if out_errors.value != 0:
            out_errors = SWCreateExplodeStepErrorE(value=out_errors.value)
            raise DocumentError(str(out_errors))

        return IExplodeStep(com_object)

    def get_root_component3(self, resolve: bool) -> IComponent2 | None:
        """
        Gets the root component for this assembly configuration.

        Reference:
        https://help.solidworks.com/2024/english/api/sldworksapi/SolidWorks.Interop.sldworks~SolidWorks.Interop.sldworks.IConfiguration~GetRootComponent3.html
        """
        com_object = self.com_object.GetRootComponent3(resolve)
        return IComponent2(com_object) if com_object else None



"""
swComponentSuppressionState_e Enumeration

Reference:
https://help.solidworks.com/2024/english/api/swconst/SOLIDWORKS.Interop.swconst~SOLIDWORKS.Interop.swconst.swComponentSuppressionState_e.html

Status: 🟢
"""

from enum import IntEnum


class SWComponentSuppressionStateE(IntEnum):
    """States for component suppression."""

    SW_COMPONENT_SUPPRESSED = 0  # Fully suppressed (recursive)
    SW_COMPONENT_LIGHTWEIGHT = 1  # Lightweight (only the component)
    SW_COMPONENT_FULLY_RESOLVED = 2  # Fully resolved (recursive)
    SW_COMPONENT_RESOLVED = 3  # Resolved (only the component)
    SW_COMPONENT_FULLY_LIGHTWEIGHT = 4  # Fully lightweight (recursive)
    SW_COMPONENT_INTERNAL_ID_MISMATCH = 5  # Internal ID mismatch



"""
swInConfigurationOpts_e Enumeration

Reference:
https://help.solidworks.com/2024/english/api/swconst/SOLIDWORKS.Interop.swconst~SOLIDWORKS.Interop.swconst.swInConfigurationOpts_e.html

Status: 🟢
"""

from enum import IntEnum


class SWInConfigurationOptsE(IntEnum):
    SW_CONFIG_PROPERTY_SUPPRESS_FEATURES = 0  # Suppress features
    SW_THIS_CONFIGURATION = 1  # This configuration only
    SW_ALL_CONFIGURATION = 2  # All configurations
    SW_SPECIFY_CONFIGURATION = 3  # Specify a configuration
    SW_LINKED_TO_PARENT = 4  # Linked to parent (valid only for derived configurations)
    SW_SPEEDPAK_CONFIGURATION = 5  # SpeedPak configuration



"""
swCustomInfoDeleteResult_e Enumeration

Reference:
https://help.solidworks.com/2024/english/api/swconst/SOLIDWORKS.Interop.swconst~SOLIDWORKS.Interop.swconst.swCustomInfoDeleteResult_e.html

Status: 🟢
"""

from enum import IntEnum


class SWCustomInfoDeleteResultE(IntEnum):
    """Result codes when deleting custom properties."""

    SW_CUSTOM_INFO_DELETE_RESULT_OK = 0  # Success
    SW_CUSTOM_INFO_DELETE_RESULT_NOT_PRESENT = 1  # Property does not exist
    SW_CUSTOM_INFO_DELETE_RESULT_LINKED_PROP = (
        2  # Property is linked and cannot be deleted
    )



"""
swGeometryToSave_e Enumeration

Reference:
https://help.solidworks.com/2024/english/api/swconst/SOLIDWORKS.Interop.swconst~SOLIDWORKS.Interop.swconst.swGeometryToSave_e.html

Status: 🟢
"""

from enum import IntEnum


class SWGeometryToSaveE(IntEnum):
    """Geometry to save options for IAdvancedSaveAsOptions::GeometryToSave."""

    SW_GEOMETRY_TO_SAVE_ALL_COMPONENTS = 0  # Save all components
    SW_GEOMETRY_TO_SAVE_EXTERIOR_FACES = 1  # Save only exterior faces
    SW_GEOMETRY_TO_SAVE_INCLUDE_SPECIFIC_COMPONENTS = 2  # Include specific components



"""
PART TOOLS // EXPORT STEP
"""

from pathlib import Path
from typing import Literal

from pyswx.api.sldworks.interfaces.i_document_specification import (
    IDocumentSpecification,
)
from pyswx.api.sldworks.interfaces.i_sldworks import ISldWorks
from pyswx.api.swconst.enumerations import SWSaveAsOptionsE
from pyswx.api.swconst.enumerations import SWSaveAsVersionE
from pyswx.tools.part_tools.open_part import open_part


def export_part(
    swx: ISldWorks,
    part_path: Path,
    export_type: Literal["step"],
    export_path: Path | None = None,
    close_document: bool = False,
    save_document: bool = True,
    document_specification: IDocumentSpecification | None = None,
) -> None:
    """
    Export a SolidWorks part to STEP format.

    Args:
        swx (ISldWorks): The SolidWorks application instance.
        part_path (Path): The path to the SolidWorks part file to be exported.
        export_type (Literal[&quot;step&quot;]): The file type of the exported data.
        export_path (Path | None, optional): The path to the step file. Same location if None. Defaults to None.
        close_document (bool, optional): Whether to close the document after export. Defaults to False.
        save_document (bool, optional): Whether to save the document before closing. Defaults to True.
        document_specification (IDocumentSpecification | None, optional): The document specification to use. Defaults to None.

    Remarks:
        The following document specification options are set:
        - document_type: SWDocumentTypesE.SW_DOC_PART
        - use_light_weight_default: True
        - light_weight: True
        - silent: True
        - ignore_hidden_components: True

        Document is saved silently.

    Raises:
        ValueError: Raised if no active document is found.
        ValueError: Raised if the active document is not a part.
        FileNotFoundError: Raised if the export fails and the step file does not exist.
    """

    part_model, _ = open_part(
        swx=swx, part_path=part_path, document_specification=document_specification
    )

    if export_path is None:
        export_path = part_model.get_path_name().with_suffix(f".{export_type}")

    part_model.extension.save_as_3(
        name=export_path,
        version=SWSaveAsVersionE.SW_SAVE_AS_CURRENT_VERSION,
        options=SWSaveAsOptionsE.SW_SAVE_AS_OPTIONS_SILENT,
        export_data=None,
        advanced_save_as_options=None,
    )

    if save_document:
        part_model.save_3(options=SWSaveAsOptionsE.SW_SAVE_AS_OPTIONS_SILENT)

    if close_document:
        swx.quit_doc(name=part_model.get_path_name())

    if not export_path.exists():
        raise FileNotFoundError(f"Failed to export STEP file: {export_path}")



"""
swDwgPaperSizes_e Enumeration

Defines standard and user-defined drawing paper sizes.

Reference:
https://help.solidworks.com/2024/english/api/swconst/SOLIDWORKS.Interop.swconst~SOLIDWORKS.Interop.swconst.swDwgPaperSizes_e.html

Status: 🟢
"""

from enum import IntEnum


class SWDwgPaperSizesE(IntEnum):
    """Drawing paper sizes."""

    SW_DWG_PAPER_ASIZE = 0  # ANSI A (landscape)
    SW_DWG_PAPER_ASIZE_VERTICAL = 1  # ANSI A (portrait)
    SW_DWG_PAPER_BSIZE = 2  # ANSI B
    SW_DWG_PAPER_CSIZE = 3  # ANSI C
    SW_DWG_PAPER_DSIZE = 4  # ANSI D
    SW_DWG_PAPER_ESIZE = 5  # ANSI E
    SW_DWG_PAPER_A4SIZE = 6  # ISO A4 (landscape)
    SW_DWG_PAPER_A4SIZE_VERTICAL = 7  # ISO A4 (portrait)
    SW_DWG_PAPER_A3SIZE = 8  # ISO A3
    SW_DWG_PAPER_A2SIZE = 9  # ISO A2
    SW_DWG_PAPER_A1SIZE = 10  # ISO A1
    SW_DWG_PAPER_A0SIZE = 11  # ISO A0
    SW_DWG_PAPERS_USER_DEFINED = 12  # User-defined paper size



"""
sw3DExperienceCfgType_e Enumeration

Reference:
https://help.solidworks.com/2024/english/api/swconst/SolidWorks.Interop.swconst~SolidWorks.Interop.swconst.sw3DExperienceCfgType_e.html

Status: 🟢
"""

from enum import IntEnum


class SW3DExperienceCfgTypeE(IntEnum):
    """Enumeration for types of configurations viewed in SOLIDWORKS Connected."""

    SW_NOT_3DEXPERIENCE_TYPE = 0  # Default SOLIDWORKS configuration
    SW_PHYSICAL_PRODUCT = 1  # Family member
    SW_REPRESENTATION = (
        2  # Break view, section view, or other representation of the physical product
    )



"""
swFileSaveError_e Enumeration

Reference:
https://help.solidworks.com/2024/english/api/swconst/SOLIDWORKS.Interop.swconst~SOLIDWORKS.Interop.swconst.swFileSaveError_e.html

Status: 🟢
"""

from enum import IntEnum


class SWFileSaveErrorE(IntEnum):
    SW_FILE_SAVE_AS_UNKNOWN = 0
    SW_GENERIC_SAVE_ERROR = 1
    SW_READ_ONLY_SAVE_ERROR = 2
    SW_FILE_NAME_EMPTY = 4  # File name cannot be empty
    SW_FILE_NAME_CONTAINS_AT_SIGN = 8  # File name cannot contain the at symbol (@)
    SW_FILE_LOCK_ERROR = 16
    SW_FILE_SAVE_FORMAT_NOT_AVAILABLE = 32  # Save As file type is not valid
    SW_FILE_SAVE_AS_DO_NOT_OVERWRITE = 128  # Do not overwrite an existing file
    SW_FILE_SAVE_AS_INVALID_FILE_EXTENSION = 256  # File name extension mismatch
    SW_FILE_SAVE_AS_NO_SELECTION = 512  # Invalid for IModelDocExtension::SaveAs
    SW_FILE_SAVE_AS_NAME_EXCEEDS_MAX_PATH_LENGTH = 2048  # File name too long (>255 characters)
    SW_FILE_SAVE_AS_NOT_SUPPORTED = 4096  # Save As is not supported or might be incomplete
    SW_FILE_SAVE_REQUIRES_SAVING_REFERENCES = 8192  # Renamed components require reference saving
    SW_FILE_SAVE_AS_DETACHED_DRAWINGS_NOT_SUPPORTED = 16384  # Detached drawing Save As not supported
    SW_FILE_SAVE_AS_BAD_EDRAWINGS_VERSION = 1024
